package AnyEvent::Subprocess::Done::Delegate;
use Moose::Role;

with 'AnyEvent::Subprocess::Delegate';

1;

__END__

=head1 NAME

AnyEvent::Subprocess::Done::Delegate - role that delegates on the Done class must implement
