package AnyEvent::Subprocess::Done::Delegate::CaptureHandle;
use Moose;

with 'AnyEvent::Subprocess::Done::Delegate';

has 'output' => ( is => 'ro', isa => 'Str', required => 1 );

__PACKAGE__->meta->make_immutable;

1;

__END__

=head1 NAME

AnyEvent::Subprocess::Done::Delegate::CaptureHandle

=head1 DESCRIPTION

Stores the output generated by a handle while the process was running

=head1 ATTRIBUTES

=head2 output

The captured output, as a string.
